package Annotations;
use strict;

use AnnotateConfig;
use DBI;
use HTML::Template;
use MIME::Lite;
use Text::Wrap;

sub new
{
	my $self = {};
	my $Config = new AnnotateConfig;

	$self->{'dbh'} = DBI->connect($Config->get('Database'),$Config->get('DatabaseUsername'),$Config->get('DatabasePassword')) or die $DBI::errstr;

	bless $self;
}

sub num_annotations()
{
	my $self = shift;
	my $node_id = shift;
	

	my $sth = $self->{'dbh'}->prepare("
		SELECT COUNT(1)
		FROM annotations
		WHERE node_id=?
	");

	$sth->execute($node_id) or die $self->{'dbh'}->errstr;

	my $num = $sth->fetchrow_array();
}

sub get_annotations()
{
	my $self = shift;
	my $node_id = shift;
	my $Config = new AnnotateConfig;
	
	my $sth = $self->{'dbh'}->prepare("
		SELECT id,Name,eMail,Date,Comment
		FROM annotations
		WHERE node_id=?
		ORDER BY id ASC
	");

	$sth->execute($node_id) or die $self->{'dbh'}->errstr;

	my @annotations = ();

	while(my ($id,$Name,$eMail,$Date,$Comment) = $sth->fetchrow_array())
	{
		my $delete_url;

		if($Config->get('AdminMode') == 1)
		{
			$delete_url = '?action=delete&id=' . $id;
		}

		push(@annotations,
			{
				'NAME' => CGI::escapeHTML($Name),
				'EMAIL' => CGI::escapeHTML(escape_email($eMail)),
				'DATE' => CGI::escapeHTML($Date),
				'COMMENT' => CGI::escapeHTML(wrap('','',$Comment)),
				'ADMIN_MODE' => ($Config->get('AdminMode') == 1),
				'DELETE_URL' => ($Config->get('AdminMode') == 1 ? $delete_url : undef)
			}
		);
	}

	return \@annotations;
}

sub add_annotation
{
	my $self = shift;
	my $entry = shift;
	my $Config = new AnnotateConfig;

	my @time = localtime(time);
	my $time = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 
		$time[5]+1900, $time[4]+1, $time[3],
		$time[2], $time[1], $time[0]);

	my $sth = $self->{'dbh'}->prepare("
		INSERT INTO annotations 
		(id,node_id,Name,eMail,Date,Comment)
		VALUES
		(NULL,?,?,?,?,?)
	");

	# ignore empty annotations silently
	if($$entry{'comment'} =~ m/^\s*$/s)
	{
		return;
	}

	# remove leading and trailing whitespace
	$$entry{'comment'} =~ s/^\s+//gs;
	$$entry{'comment'} =~ s/\s+$//gs;

	$sth->execute($$entry{'node_id'},$$entry{'name'},$$entry{'eMail'},$time,$$entry{'comment'});

	my $body = new HTML::Template('filename' => $Config->get('TemplateRoot')
			 . '/' . $Config->get('MailTemplate'));
	$body->param('NAME',$$entry{'name'});
	$body->param('EMAIL',$$entry{'eMail'});
	$body->param('COMMENT',wrap('','',$$entry{'comment'}));
	$body->param('NODE_ID',$$entry{'node_id'});

	if($Config->get('eMailAddress') ne "")
	{
		my $msg = new MIME::Lite(
			'From' => 'Annotate <' . $Config->get('eMailAddress') . '>',
			'To' => $Config->get('eMailAddress'),
			'Subject' => $Config->get('eMailSubject'),
			'Data' => $body->output()
		);
		$msg->send();
	}
}

sub delete_annotation
{
	my $self = shift;
	my $id = shift;

	my $sth = $self->{'dbh'}->prepare("
		DELETE FROM annotations 
		WHERE id = ?
	");

	$sth->execute($id);
}

sub escape_email
{
	my $string = shift;

	$string =~ s/.*?(\S+)@(\S+).*/$1 at $2/g;
	$string =~ s/\./ dot /g;

	$string;
}

1;
