# the Hide module
# prints a navigation header and footer and strips the annotation anchors
package Output::Hide;
use strict;
use AnnotateConfig;
use HTML::Parser;
use HTML::Template;

sub new
{
	my $self = {};
	my $Config = new AnnotateConfig;

	$self->{'navigationtpl'} = new HTML::Template('filename' => $Config->get('TemplateRoot') . '/' . $Config->get('NavigationTemplate'));

	bless($self);
	return($self);
}

# what is done here is to parse the HTML tree of the input document, look
# for the Annotate-specific tags and output links/annotations/input forms
# at these places
# the HTML tree is processed in a depth-first way in order to maintain
# the nested structure of the Annotate anchors
sub print
{
	my $self = shift;
	my $string = shift;
	my $selected_node_id = shift; # the selected node id, if one is to be annotated

	my $parser = new HTML::Parser(
		api_version => 3,
		default_h => [ sub { $self->tag_handler(@_, $selected_node_id) }, 'event, tagname, attr, text' ]
	);

	$$self{'depth'} = 0;
	$$self{'stack'} = [];
	$$self{'node_id'} = [];

	$parser->parse($string);

	print $$self{'stack'}[0];
}

sub tag_handler
{
	my $self = shift;
	my $event = shift;
	my $tagname = shift;
	my $attr = shift;
	my $text = shift;
	my $selected_node_id = shift;

	if($tagname eq 'annotate')
	{
		if($event eq 'start')
		{
			# an <annotate> section begins: increment depth
			# and save the node_id
			$$self{'depth'}++;
			$$self{'node_id'}[$$self{'depth'}] = $$attr{'node_id'};
		}
		elsif($event eq 'end')
		{
			# an <annotate> section ends: process the contents
			# of the section using $self->annotation()
			# then append the result to the result of depth-1
			# and decrement the current depth
			$$self{'stack'}[$$self{'depth'}-1] .= $self->annotation($$self{'node_id'}[$$self{'depth'}], $$self{'stack'}[$$self{'depth'}], $selected_node_id);
			undef $$self{'stack'}[$$self{'depth'}];
			$$self{'depth'}--;
		}
	}
	elsif($tagname eq 'annotate-navigation')
	{
		if($event eq 'start')
		{
			$$self{'stack'}[$$self{'depth'}] .= $self->navigation($$attr{'type'});
		}
	}
	else
	{
		$$self{'stack'}[$$self{'depth'}] .= $text;
	}
}

sub annotation
{
	my $self = shift;
	my $node_id = shift;
	my $content = shift;

# this is the hide module, so return the content only
	return $content;
}

sub navigation
{
	my $self = shift;
	my $type = shift; # in case we ever need to distinguish header and footer

	$self->{'navigationtpl'}->param('HIDE' => "?mode=Hide");
	$self->{'navigationtpl'}->param('LINK' => "?mode=Link");
	$self->{'navigationtpl'}->param('INLINE' => "?mode=Inline");

	$self->{'navigationtpl'}->output();
}

1;
