#!/usr/bin/perl

@vars = ("stylesheet","tags","documentroot","adminusername","adminpassword","dbi","dbiusername","dbipassword","mimetypes");

$configtype = "b";
$stylesheet = "/usr/local/share/xsl/docbook/html/chunk.xsl";
$tags = "para simpara programlisting note important warning caution tip";
$documentroot = "/usr/local/share/annotate";
$adminusername = "admin";
$adminpassword = generate_password();
$dbi = "DBI:SQLite:annotations/annotations.db";
$dbiusername = "";
$dbipassword = "";
$mimetypes = "/etc/httpd/mime.types";

$query{'sqlite'} = <<END
CREATE TABLE annotations (
  id INTEGER PRIMARY KEY,
  node_id TEXT,
  Name TEXT,
  eMail TEXT,
  Date TEXT,
  Comment TEXT
)
END
;
$query{'mysql'} = <<END
CREATE TABLE annotations (
  id int(10) unsigned NOT NULL auto_increment,
  node_id varchar(255) NOT NULL default '',
  Name varchar(255) NOT NULL default '',
  eMail varchar(255) NOT NULL default '',
  Date datetime NOT NULL default '0000-00-00 00:00:00',
  Comment text NOT NULL,
  PRIMARY KEY  (id)
) TYPE=MyISAM;
END
;

while($ok ne "yes")
{
	print "Is this your (a)uthoring station, your (w)eb server or (b)oth? [$configtype]";
	set_value("configtype");

	if(lc $configtype eq "a")
	{
		$do_authoring = 1;
		$do_webserver = 0;
	}
	elsif(lc $configtype eq "w")
	{
		$do_authoring = 0;
		$do_webserver = 1;
	}
	else
	{
		$do_authoring = 1;
		$do_webserver = 1;
	}

	if($do_authoring)
	{
		print "Where is your DocBook XSL stylesheet? [$stylesheet] ";
		set_value("stylesheet");

		print "Which DocBook elements do you want to annotate? [$tags] ";
		set_value("tags");
	}

	if($do_webserver)
	{
		check_module('CGI');
		check_module('DBI');
		check_module('HTML::Parser');
		check_module('HTML::Template');
		check_module('MIME::Lite');
		check_module('Text::Wrap');
		check_module('XML::Simple');

		print "In which directory can Annotate find the HTML version of your document? [$documentroot] ";
		set_value("documentroot");

		print "Please enter the username for the web administration [$adminusername] ";
		set_value("adminusername");

		print "Please enter the password for the web administration [$adminpassword] ";
		set_value("adminpassword");

		print "Please provide me with a DBI data source string. [$dbi] ";
		set_value("dbi");

		print "If needed, please provide the database username. [$dbiusername] ";
		set_value("dbiusername");

		print "If needed, please provide the database password. [$dbipassword] ";
		set_value("dbipassword");

		print "If possible, please provide the path of Apache's mime.types database. [$mimetypes] ";
		set_value("mimetypes");
	}

	print "\n";
	if($do_authoring)
	{
		print "DocBook XSL stylesheet: $stylesheet\n";
		print "Tags to annotate: $tags\n";
	}

	if($do_webserver)
	{
		print "DocumentRoot: $documentroot\n";
		print "Web Username: $adminusername\n";
		print "Web Password: $adminpassword\n";
		print "DBI source: $dbi\n";
		print "DBI Username: $dbiusername\n";
		print "DBI Password: $dbipassword\n";
		print "MIME Database: $mimetypes\n";
	}

	print "\neverything ok? [Y/n] ";
	set_yesno("ok");
}

$tags =~ s/\s+/|/g;

open(CONF,">annotate.conf") or die "Could not open output file: $!";
while($line = <DATA>)
{
	foreach $var (@vars)
	{
		$ucvar = "__" . uc $var . "__";
		$line=~s/$ucvar/$$var/g;
	}
	print CONF $line;
}
close(CONF);

$dbi =~ m/:(.*):/;
$database = lc $1;

if($do_webserver)
{
	for(keys %query)
	{
		if($database eq $_)
		{
			print $query{$_};

			print "You specified $database as the DBI source. Shall I try and create a table using the query above? [Y/n] ";

			set_yesno("ok");

			if($ok eq 'yes')
			{
				require DBI;

				# hint for SQLite and other file-based
				# databases: DBI data source is relative
				# to the cgi-bin directory
				chdir 'cgi-bin';

				my $dbi = DBI->connect($dbi, $dbiusername, $dbipassword) or die $DBI::errstr;
				my $sth = $dbi->prepare($query{$_});
				$sth->execute;

				break;
			}
		}
	}
}

sub set_value($)
{
	my $name = shift;
	my $value = <STDIN>;
   
	chomp($value);

	if($value ne "")
	{
		$$name = $value;
	}
}


print <<END
$0 created the configuration file annotate.conf.

Please move it to /usr/local/etc or consult the Annotate documentation
for further information.
END
;

sub set_yesno($)
{
	my $name = shift;
	my $value = <STDIN>;
	chomp($value);

	if($value eq "y" or $value eq "Y" or $value eq "")
	{
		$$name = "yes";
	}
	else
	{
		$$name = "no";
	}
}

sub generate_password
{
	my $chars = 'abcdefghkmnpqrstuvwxyz23456789';
	my $result = '';

	for(my $i=1; $i <= 8; $i++)
	{
		$result .= substr($chars,(int rand length $chars),1);
	}

	$result;
}

sub check_module
{
	my $module = shift;

	print "checking for module $module ... ";

	if(eval "require $module")
	{
		print "ok\n";
	}
	else
	{
		print "not available!\n";
		print "\nThe $module module is missing on your system.\n";
		print "In order to install it, you can execute this command (as root):.\n\n";
		print "perl -MCPAN -e 'install $module'\n\n";
		
		exit;
	}
}
__END__
<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<!--
 Part 1: set the path to your docbook stylesheet here
-->
<xsl:import href="__STYLESHEET__"/>

<!--
 Part 2: set the value of the match attribute to the list of
  tags that you want to add annotations to
-->
<xsl:template match="__TAGS__">
  <xsl:call-template name="annotate"/>
</xsl:template>

<!--
 Part 3: configure these variables to suit your needs
-->

<!-- the DocumentRoot for the DocBook-Document to annotate -->
<xsl:variable name="DocumentRoot">__DOCUMENTROOT__</xsl:variable>

<!-- the file to show if the requested file is a directory -->
<xsl:variable name="DirectoryIndex">index.html</xsl:variable>

<!-- the eMail address to send new annotations to -->
<xsl:variable name="eMailAddress"></xsl:variable>

<!-- the subject to use for mails sent after a new annotation -->
<xsl:variable name="eMailSubject"></xsl:variable>

<!-- the root directory where the templates can be found -->
<xsl:variable name="TemplateRoot">templates</xsl:variable>

<!-- the name of the template which provides the navigation bar -->
<xsl:variable name="NavigationTemplate">navigation.tpl</xsl:variable>

<!-- the name of the template which provides the links -->
<xsl:variable name="LinkTemplate">links.tpl</xsl:variable>

<!-- the name of the template which provides an entry form for a new annotation -->
<xsl:variable name="AnnotateTemplate">annotate.tpl</xsl:variable>

<!-- the name of the template which displays annotations already made -->
<xsl:variable name="ViewTemplate">view.tpl</xsl:variable>

<!-- the name of the template for the mail to be sent on new annotations -->
<xsl:variable name="MailTemplate">mail.tpl</xsl:variable>

<!-- the name of the template for the login page -->
<xsl:variable name="LoginTemplate">login.tpl</xsl:variable>

<!-- the DBI database handle (i.e., the parameter for DBI->connect()) -->
<xsl:variable name="Database">__DBI__</xsl:variable>

<!-- the DBI database username -->
<xsl:variable name="DatabaseUsername">__DBIUSERNAME__</xsl:variable>

<!-- the DBI database password -->
<xsl:variable name="DatabasePassword">__DBIPASSWORD__</xsl:variable>

<!-- the administration username -->
<xsl:variable name="AdminUsername">__ADMINUSERNAME__</xsl:variable>

<!-- the administration password -->
<xsl:variable name="AdminPassword">__ADMINPASSWORD__</xsl:variable>

<!-- the default output mode for annotations. valid possibilities are:
	   Hide - hides annotations
	   Inline - shows annotations inline
	   Link - shows links to the annotations. the annotations will open in a separate window
-->
<xsl:variable name="DefaultOutputMode">Link</xsl:variable>

<!-- optional: Apache's mime.types database to determine Content-Types -->
<xsl:variable name="MimeTypes">__MIMETYPES__</xsl:variable>

<!--
 usually there is nothing more to change after this point
-->

<xsl:template name="user.header.navigation">
	<annotate-navigation type="header"/>
</xsl:template>

<xsl:template name="user.footer.navigation">
	<annotate-navigation type="footer"/>
</xsl:template>

<xsl:template name="annotate">
  <xsl:param name="node" select="."/>
  <xsl:variable name="node_id">
	<xsl:call-template name="object.id">
	  <xsl:with-param name="object" select="$node"/>
	</xsl:call-template>
  </xsl:variable>
  <annotate node_id="{$node_id}">
    <xsl:apply-imports/>
  </annotate>
</xsl:template>


</xsl:stylesheet>
