#!/usr/bin/perl

# this is the list of annotatable elements
$annotatable = 'simpara|programlisting|note|important|warning|caution|tip|para';

use XML::Parser;
use Data::Dumper;

map { $is_annotatable{$_} = 1} (split /\|/, $annotatable);

while($inputfile = shift @ARGV)
{
	# this is a two-pass process
	#  1. collect IDs already in use
	#  2. assign new ID to every annotatable element lacking one

	my $id_collector = new XML::Parser(Handlers => { Start => \&collect_handler });
	$id_collector->parsefile($inputfile);

	# by now, assigned IDs are in %in_use

	$id_generator = new XML::Parser(
		Handlers => { Default => \&default_handler, Start => \&start_handler, End => \&end_handler },
		NoExpand => 1
	);

	$id_generator->parsefile($inputfile);
}

sub collect_handler
{
	my $expat = shift;
	my $element = shift;

	if($is_annotatable{$element})
	{
		while(my $attribute = shift @_)
		{
			my $value = shift @_;

			if($attribute eq 'id')
			{
				$in_use{$value} = 1;
			}			
		}
	}
}

sub default_handler
{
	my $expat = shift;
	my $string = shift;

	print $string;
}

sub start_handler
{
	my $expat = shift;
	my $element = shift;

	print "<$element";

	my $id_exists = 0;

	while(my $attribute = shift @_)
	{
		print(" $attribute=\"" . (shift @_) . "\"");

		if($attribute eq 'id')
		{
			$id_exists = 1;
		}
	}

	if($is_annotatable{$element} && !$id_exists)
	{
		print " id=\"" . generate_id() . "\"";
	}

	print ">";
}

sub end_handler
{
	my $expat = shift;
	my $element = shift;

	print "</$element>";
}

sub generate_id
{
	my $id;

	do {
		$id = 'annotateid-' . sprintf("%x-%x",time,int(rand(65535)));
	} while($in_use{$id});

	$in_use{$id} = 1;

	return $id;
}
